/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.actor.impl.io.fileload;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.io.IOException;
import java.nio.ByteBuffer;
import it.tidalwave.util.Task;
import javax.annotation.concurrent.NotThreadSafe;
import lombok.Getter;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@NotThreadSafe
class FileLoaderActorStats implements FileLoaderActorStatsMBean
  {
    private static final long MEGA = 1000 * 1000;
    
    private long previousTime;
    
    private long startTime;
    
    private long baseTime;
    
    @Getter
    private long inactiveTime;
    
    @Getter
    private long latestInactiveTime;
    
    @Getter
    private long latestElapsedTime;
    
    @Getter
    private long latestSpeed; // in bytes / sec
    
    @Getter
    private int invocationCount;
    
    @Getter
    private long loadedByteCount;
    
    private long cumulativeElapsedTime;
    
    private boolean resetPending;
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Override
    public void reset()
      {
        resetPending = true;  
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    public ByteBuffer execute (final @Nonnull Task<ByteBuffer, IOException> task)
      throws IOException
      {
        ByteBuffer result = null;
        invocationCount++;
        
        try
          {
            start();
            result = task.run();
            return result;
          }
        finally
          {
            stop();
            loadedByteCount += (result == null) ? 0 : result.limit();
            latestSpeed = (result == null) ? 0 : (1000L * result.limit()) / Math.max(1, latestElapsedTime);
          }
      }

    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Nonnegative @Override
    public double getEfficiency()
      {
        return 1.0 - inactiveTime / (double)((System.currentTimeMillis() - baseTime));  
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Override
    public long getSpeed() 
      {
        return (1000L * loadedByteCount) / Math.max(1, cumulativeElapsedTime);
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    private void start()
      {
        startTime = System.currentTimeMillis();
        
        if (previousTime == 0)
          {
            previousTime = startTime;  
          }
        
        if (baseTime == 0)
          {
            baseTime = startTime; 
          }
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    private void stop()
      {
        latestInactiveTime = startTime - previousTime;
        inactiveTime += latestInactiveTime;
        previousTime = System.currentTimeMillis();
        latestElapsedTime = previousTime - startTime;
        cumulativeElapsedTime += latestElapsedTime;
        
        if (resetPending)
          {
            resetPending = false;   
            previousTime = 0;
            startTime = 0;
            baseTime = 0;
            inactiveTime = 0;
            latestInactiveTime = 0;
            latestElapsedTime = 0;
            latestSpeed = 0;
            invocationCount = 0;
            loadedByteCount = 0;
            cumulativeElapsedTime = 0;
          }
      }

    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Override @Nonnull
    public String toString()
      {
        return String.format("FileLoaderActorStats[load speed: %d MB/s - was inactive for %d msec - efficiency: %.0f%%]", 
                             latestSpeed / MEGA, latestInactiveTime, getEfficiency() * 100);
      }
  }
